import wrapReadModel from './wrap-read-model';
import wrapViewModel from './wrap-view-model';
import { CreateQueryOptions } from './types';
import { OMIT_BATCH, STOP_BATCH } from './batch';

const dispose = async models => {
  for (const modelName of Object.keys(models)) {
    await models[modelName].dispose();
  }
};

const interopApi = async (models, key, ...args) => {
  if (args.length !== 1 || Object(args[0]) !== args[0]) {
    throw new TypeError(`Invalid resolve-query method "${key}" arguments ${JSON.stringify(args)}`);
  } // eslint-disable-next-line prefer-const


  let {
    eventSubscriber,
    modelName,
    ...parameters
  } = args[0];

  if (eventSubscriber == null && modelName == null) {
    throw new Error(`Either "eventSubscriber" nor "modelName" is null`);
  } else if (modelName == null) {
    modelName = eventSubscriber;
  }

  if (models[modelName] == null) {
    const error = new Error(`Read/view model "${modelName}" does not exist`);
    error.code = 422;
    throw error;
  }

  const method = models[modelName][key];

  if (typeof method !== 'function') {
    throw new TypeError(`Model "${modelName}" does not implement method "${key}"`);
  }

  return await method(parameters);
};

const createQuery = params => {
  const models = {};
  const {
    viewModelsInterop,
    readModelsInterop,
    ...imports
  } = params;

  for (const model of Object.values(readModelsInterop)) {
    models[model.name] = wrapReadModel({
      interop: model,
      ...imports
    });
  }

  for (const model of Object.values(viewModelsInterop)) {
    models[model.name] = wrapViewModel({
      interop: model,
      ...imports
    });
  }

  const read = interopApi.bind(null, models, 'read');
  const api = new Proxy(read, {
    get(_, key) {
      if (key === 'bind' || key === 'apply' || key === 'call') {
        return read[key].bind(read);
      } else if (key === 'dispose') {
        return dispose.bind(null, models);
      } else {
        return interopApi.bind(null, models, key);
      }
    },

    set() {
      throw new TypeError(`Resolve-query API is immutable`);
    }

  });
  return api;
};

export { CreateQueryOptions, OMIT_BATCH, STOP_BATCH };
export default createQuery;
//# sourceMappingURL=index.js.map