"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.uploadFormData = exports.upload = void 0;

var _fs = _interopRequireDefault(require("fs"));

var _request = _interopRequireDefault(require("request"));

var _uuid = require("uuid");

var _crypto = _interopRequireDefault(require("crypto"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const createPresignedPut = async (pool, dir) => {
  const uploadId = (0, _uuid.v4)();
  const uploadUrl = `http://localhost:3000/uploader?dir=${dir}&uploadId=${uploadId}`;
  return {
    uploadUrl,
    uploadId
  };
};

const upload = (uploadUrl, filePath) => {
  const fileSizeInBytes = _fs.default.statSync(filePath).size;

  const fileStream = _fs.default.createReadStream(filePath);

  return new Promise((resolve, reject) => _request.default.put({
    headers: {
      'Content-Length': fileSizeInBytes
    },
    uri: uploadUrl,
    body: fileStream
  }, (error, _, body) => {
    error ? reject(error) : body ? reject(body) : resolve();
  }));
};

exports.upload = upload;

const createPresignedPost = async (pool, dir) => {
  const uploadId = (0, _uuid.v4)();
  const form = {
    url: `http://localhost:3000/uploader?dir=${dir}&uploadId=${uploadId}`,
    fields: {}
  };
  return {
    form,
    uploadId
  };
};

const uploadFormData = (form, filePath) => {
  const fileStream = _fs.default.createReadStream(filePath);

  return new Promise((resolve, reject) => _request.default.post({
    url: form.url,
    formData: {
      file: fileStream
    }
  }, error => {
    error ? reject(error) : resolve();
  }));
};

exports.uploadFormData = uploadFormData;

const createToken = ({
  secretKey
}, {
  dir,
  expireTime = 3600
}) => {
  const payload = Buffer.from(JSON.stringify({
    dir,
    expireTime: Date.now() + expireTime * 1000
  })).toString('base64').replace(/=/g, '');

  const signature = _crypto.default.createHmac('md5', secretKey).update(payload).digest('hex');

  return `${payload}*${signature}`;
};

const createUploader = pool => {
  const {
    directory,
    bucket,
    secretKey
  } = pool;
  return Object.freeze({
    createPresignedPut: createPresignedPut.bind(null, pool),
    createPresignedPost: createPresignedPost.bind(null, pool),
    createToken: createToken.bind(null, pool),
    upload: upload,
    uploadFormData: uploadFormData,
    directory,
    bucket,
    secretKey
  });
};

const getSignedPut = async (adapter, dir) => await adapter.createPresignedPut(dir);

const getSignedPost = async (adapter, dir) => await adapter.createPresignedPost(dir);

const initUploader = async resolve => {
  if (resolve.assemblies.uploadAdapter != null) {
    // TODO: provide support for custom uploader adapter
    const createUploadAdapter = resolve.assemblies.uploadAdapter;
    const uploader = createUploader(createUploadAdapter());
    process.env.RESOLVE_UPLOADER_CDN_URL = 'http://localhost:3000/uploader';
    Object.assign(resolve, {
      uploader: {
        getSignedPut: getSignedPut.bind(null, uploader),
        getSignedPost: getSignedPost.bind(null, uploader),
        uploadPut: uploader.upload,
        uploadPost: uploader.uploadFormData,
        createToken: uploader.createToken,
        directory: uploader.directory,
        bucket: uploader.bucket,
        secretKey: uploader.secretKey
      }
    });
  }
};

var _default = initUploader;
exports.default = _default;
//# sourceMappingURL=init-uploader.js.map