"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _debugLevels = _interopRequireDefault(require("@resolve-js/debug-levels"));

var _events = _interopRequireDefault(require("events"));

var _http = _interopRequireDefault(require("http"));

var _ws = _interopRequireDefault(require("ws"));

var _uuid = require("uuid");

var _querystring = _interopRequireDefault(require("querystring"));

var _jsonwebtoken = _interopRequireDefault(require("jsonwebtoken"));

var _createPubsubManager = _interopRequireDefault(require("./create-pubsub-manager"));

var _getRootBasedUrl = _interopRequireDefault(require("../common/utils/get-root-based-url"));

var _getSubscribeAdapterOptions = _interopRequireDefault(require("./get-subscribe-adapter-options"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const log = (0, _debugLevels.default)('resolve:runtime:local-subscribe-adapter');
let eventstoreAdapter = null;

const createWebSocketConnectionHandler = resolve => (ws, req) => {
  const {
    pubsubManager
  } = resolve;
  const queryString = req.url.split('?')[1];

  const {
    token,
    deploymentId
  } = _querystring.default.parse(queryString);

  const connectionId = (0, _uuid.v4)();
  let eventTypes = null;
  let aggregateIds = null;

  try {
    void ({
      eventTypes,
      aggregateIds
    } = _jsonwebtoken.default.verify(token, deploymentId));
  } catch (error) {
    throw new Error('Permission denied, invalid token');
  }

  const publisher = event => ws.send(event);

  pubsubManager.connect({
    client: publisher,
    connectionId,
    eventTypes,
    aggregateIds
  });

  const dispose = () => {
    pubsubManager.disconnect({
      connectionId
    });
    ws.close();
  };

  const handler = createWebSocketMessageHandler(resolve, ws, connectionId);
  ws.on('message', handler);
  ws.on('close', dispose);
  ws.on('error', dispose);
};

const createWebSocketMessageHandler = ({
  pubsubManager
}, ws, connectionId) => async message => {
  try {
    const {
      eventTypes,
      aggregateIds
    } = pubsubManager.getConnection({
      connectionId
    });
    const parsedMessage = JSON.parse(message);

    switch (parsedMessage.type) {
      case 'pullEvents':
        {
          const {
            events,
            cursor
          } = await eventstoreAdapter.loadEvents({
            eventTypes,
            aggregateIds,
            limit: 1000000,
            eventsSizeLimit: 124 * 1024,
            cursor: parsedMessage.cursor
          });
          ws.send(JSON.stringify({
            type: 'pullEvents',
            payload: {
              events,
              cursor
            }
          }));
          break;
        }

      default:
        {
          throw new Error(`The '${parsedMessage.type}' message type is unknown`);
        }
    }
  } catch (error) {
    // eslint-disable-next-line no-console
    console.warn(`Error while handling message from websocket: ${error != null && error.message != null ? `${error.message} ${error.stack}` : JSON.stringify(error)}`);
  }
};

const initInterceptingHttpServer = resolve => {
  const {
    server: baseServer,
    websocketHttpServer
  } = resolve;
  const websocketBaseUrl = (0, _getRootBasedUrl.default)(resolve.rootPath, '/api/websocket');
  const interceptingEvents = ['close', 'listening', 'request', 'upgrade', 'error', 'connection'];

  const interceptingEventListener = (eventName, listeners, ...args) => {
    const requestUrl = args[0] != null && args[0].url != null ? String(args[0].url) : '';

    if (requestUrl.startsWith(websocketBaseUrl)) {
      websocketHttpServer.emit(eventName, ...args);
    } else {
      for (const listener of listeners) {
        listener.apply(baseServer, args);
      }
    }
  };

  for (const eventName of interceptingEvents) {
    const listeners = baseServer.listeners(eventName).slice(0);
    baseServer.removeAllListeners(eventName);
    const listener = interceptingEventListener.bind(null, eventName, listeners);
    baseServer.on(eventName, listener);
  }
};

const initWebSocketServer = async resolve => {
  try {
    const websocketServer = new _ws.default.Server({
      path: (0, _getRootBasedUrl.default)(resolve.rootPath, '/api/websocket'),
      server: resolve.websocketHttpServer
    });
    const connectionHandler = createWebSocketConnectionHandler(resolve);
    websocketServer.on('connection', connectionHandler);
  } catch (error) {
    log.warn('Cannot init WebSocket server: ', error);
  }
};

const createSocketHttpServer = () => {
  const socketServer = new _events.default();
  Object.setPrototypeOf(socketServer, _http.default.Server.prototype);
  Object.defineProperty(socketServer, 'listen', {
    value: () => {}
  });
  return socketServer;
};

const initWebsockets = async resolve => {
  const pubsubManager = (0, _createPubsubManager.default)();
  const websocketHttpServer = createSocketHttpServer();
  eventstoreAdapter = await resolve.assemblies.eventstoreAdapter();

  const sendReactiveEvent = async event => {
    await resolve.pubsubManager.dispatch({
      topicName: event.type,
      topicId: event.aggregateId,
      event
    });
  };

  Object.defineProperties(resolve, {
    getSubscribeAdapterOptions: {
      value: _getSubscribeAdapterOptions.default
    },
    pubsubManager: {
      value: pubsubManager
    },
    websocketHttpServer: {
      value: websocketHttpServer
    },
    sendReactiveEvent: {
      value: sendReactiveEvent
    }
  });
  await initWebSocketServer(resolve);
  await initInterceptingHttpServer(resolve);
};

var _default = initWebsockets;
exports.default = _default;
//# sourceMappingURL=init-websockets.js.map